import yaml 


class Config: 
    """
    A class containing the elements defined in the recipe 
    file. 
    ...

    Attributes
    ----------
    (...): All the attributes defined in the recipe
           deleting the first level of identation of the 
           YML file and combining all of them together
    arch : dict
        Dict containing the esarchive structure defined in
        the arch_conf file         

    """

    def __init__(self, config_file): 
        """
        Parameters
        ----------
        config_file : str
            Path to the recipe 
        """

        self.set_parameters(config_file) 
        self.arch = Archive(self.arch_conf)

    def set_parameters(self, config_file): 
        """
        Loads all the parameters present in the yml file
        as attributes of the class

        Parameters
        ----------
        config_file : str
            Path to the recipe 
        """

        with open(config_file) as f:     
            conf = yaml.load(f, Loader=yaml.FullLoader)
            for box, vals in conf.items():
                for k, v in vals.items(): 
                    setattr(self, k, v) 

class Archive: 
    """
    A class containing the structure of esarchive and
    all the systems present.
    ...

    Attributes
    ----------
    src: str with the path to the archive root
    (system name): All the attributes defining the system

    """
    def __init__(self, arch_file): 
        self.set_parameters(arch_file) 

    def set_parameters(self, arch_file): 
        """
        Loads all the parameters present in the yml file
        as attributes of the class

        Parameters
        ----------
        arch_file : str
            path to the archive yml file 
        """
        with open(arch_file) as f:     
            file = yaml.load(f, Loader=yaml.FullLoader)
            for k, v in file['dirs'].items(): 
                setattr(self, k, v) 
    
    def get_system(self,system, tempfreq):
        """
        Parameters
        ----------
        system : str
            name of the system to retrieve 
        tempfreq: str 
            temporal frequency of the data (daily_mean, daily, monthly_mean)

        Returns
        ----------
        system_dict : dict
            dict with system info 
        """
        system_dict = getattr(self,system)
        system_dict = {"dir": self.esarchive + system_dict['src'] + tempfreq + '/',
                       "subdirs": system_dict[tempfreq],
                       "type": system_dict['type'],
                       "ensemble": system_dict['ensemble']}

        return(system_dict)