
library(CSTools)
library(startR)
library(s2dv)
library(lubridate)
library(multiApply)
source('/esarchive/scratch/jramon/GitLab_jramon/downscaling/csdownscale/R/Interpolation.R')
source('/esarchive/scratch/jramon/GitLab_jramon/downscaling/csdownscale/R/Analogs.R')
source('/esarchive/scratch/jramon/GitLab_jramon/downscaling/csdownscale/R/Utils.R')

plotpath <- '/esarchive/scratch/jramon/downscaling/plots/ip'
#target_grid <- '/esarchive/recon/ecmwf/era5/daily/tasmax-r1440x721cds/tasmax_201702.nc'

#lonmin <- -22
#lonmax <- 45
#latmin <- 27
#latmax <- 72
lonmin <- -11.5
lonmax <- 5.35
latmin <- 35.1
latmax <- 44.1
extra <- 1

#sdates <- c('20000201','20010201','20020201','20030201','20040201','20050201','20060201','20070201')
#sdates <- format(seq(ymd("20000101"), ymd("20071201"), '1 month'), "%Y%m%d")
sdates_exp <- format(ymd("20000501") + rep(years(0:2), each=1),"%Y%m%d")
#sdates_obs  <- format(ymd("20000401") + months(0:2) + rep(years(0:4), each=3),"%Y%m")
sdates_obs  <- format(ymd("20000501") + rep(years(0:2), each=1),"%Y%m")

#---------------------------
# Observations
#---------------------------
# dim(obs) <- c('sdate', 'smonth', ...)
obs <- startR::Start(dat = '/esarchive/recon/ecmwf/era5/daily_mean/$var$_f1h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1:28), lat = values(list(latmin - extra, latmax + extra)), 
                     sdate = sdates_obs, lat_reorder = Sort(decreasing = FALSE),
                     lon = values(list(lonmin - extra, lonmax + extra)), lon_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'), 
                     lat = c('lat', 'latitude')), return_vars = list(lat = 'dat', lon = 'dat'),
                     num_procs = 1, retrieve = TRUE)
obs <- s2dv_cube(obs, lat = attr(obs, "Variables")$dat1$lat, lon = attr(obs, "Variables")$dat1$lon,
                 source_files = attr(obs, "Files")[1,1,])
attr(obs$lon,"first_lon") <- obs$lon[1]
attr(obs$lon,"last_lon") <- obs$lon[length(obs$lon)]
attr(obs$lat,"first_lat") <- obs$lat[1]
attr(obs$lat,"last_lat") <- obs$lat[length(obs$lat)]

# Create window to look for analogues
dim(obs$data) <- c(data = 1, var = 1, time = 28, lat = 39, smonth = 3, sdate = 7, lon = 67) # correct
obs_new <- Apply(obs$data, target_dims = list(c("time", "smonth")), fun = as.vector, output_dims = "time")$output1
wlen <- 7

result <- array(NA, dim = c(window = 2*wlen + 1, time  = 28, lat = 39, sdate = 7, lon = 67))
for (i in 1:28) {
  result[,i,,,] <- obs_new[(28 + i - wlen):(28 + i + wlen),1,1,,,]
}

obs$data <- result

# > dim(obs$data)
#  data    var   time    lat smonth  sdate    lon 
#     1      1     28     39      3      7     67 
# > dim(result)
#  window   time    lat  sdate    lon 
#      15     28     39      7     67 

#---------------------------
# Model
#---------------------------
exp <- startR::Start(dat = '/esarchive/exp/ecmwf/system5c3s/daily_mean/$var$_f6h/$var$_$sdate$.nc',
                     var = 'tas', time = indices(1:28), member = indices(1:3), sdate = sdates_exp,
                     lat = values(list(latmin, latmax)), lat_reorder = Sort(decreasing = FALSE), 
                     lon = values(list(lonmin, lonmax)), lon_reorder = CircularSort(-180, 180),
                     synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'), 
                     lat = c('lat', 'latitude'), member = c('member','ensemble')),
                     return_vars = list(lat = 'dat', lon = 'dat'),
                     num_procs = 1, retrieve = TRUE)
exp <- s2dv_cube(exp, lat = attr(exp, "Variables")$dat1$lat, lon = attr(exp, "Variables")$dat1$lon,
                 source_files = attr(exp, "Files")[1,1,])
attr(exp$lon,"first_lon") <- exp$lon[1]
attr(exp$lon,"last_lon") <- exp$lon[length(exp$lon)]
attr(exp$lat,"first_lat") <- exp$lat[1]
attr(exp$lat,"last_lat") <- exp$lat[length(exp$lat)]


# test examples
ana_mean <- Analogs(exp = exp, obs = obs, fun_analog = "mean", ncores = 4)
ana_wmean <- Analogs(exp = exp, obs = obs, fun_analog = "wmean", ncores = 4)
ana_max <- Analogs(exp = exp, obs = obs, fun_analog = "max", ncores = 4)
ana_min <- Analogs(exp = exp, obs = obs, fun_analog = "min", ncores = 4)
ana_median <- Analogs(exp = exp, obs = obs, fun_analog = "median", ncores = 4)

# plot examples
s2dv::PlotEquiMap(ana_mean$data[,,1,1,1,1,1], lat = ana_mean$lat, lon = ana_mean$lon, filled.continents = F,
                  toptitle = 'Analogs mean', brks = seq(275,288,1), height = 8, width = 10, 
                  fileout = file.path(plotpath, "ip_ana_mean_2.png"))
s2dv::PlotEquiMap(ana_wmean$data[1,,,1,1,1,1], lat = ana_wmean$lat, lon = ana_wmean$lon, filled.continents = F,
                  toptitle = 'Analogs weighted mean', brks = seq(275,288,1), height = 8, width = 10,
                  fileout = file.path(plotpath, "ip_ana_wmean.png"))
s2dv::PlotEquiMap(ana_max$data[1,,,1,1,1,1], lat = ana_max$lat, lon = ana_max$lon, filled.continents = F,
                  toptitle = 'Analogs max', brks = seq(275,288,1), height = 8, width = 10,
                  fileout = file.path(plotpath, "ip_ana_max.png"))
s2dv::PlotEquiMap(ana_median$data[1,,,1,1,1,1], lat = ana_max$lat, lon = ana_max$lon, filled.continents = F,
                  toptitle = 'Analogs median', brks = seq(275,288,1), height = 8, width = 10,
                  fileout = file.path(plotpath, "ip_ana_median.png"))
s2dv::PlotEquiMap(ana_min$data[1,,,1,1,1,1], lat = ana_max$lat, lon = ana_max$lon, filled.continents = F,
                  toptitle = 'Analogs min', brks = seq(275,288,1), height = 8, width = 10,
                  fileout = file.path(plotpath, "ip_ana_min.png"))










